<?php $app_version = "v13.0.0.31"; $app_revdate = "B230715-103546";
/*************************************************************************
Folio Multipage document handler for TNG
Written by William Bisbee, http://www.bisbeefamily.com.
Displays a folder of page files as a multi-page document.

version 13.0.0.18 Revised to work with TNGv13
version 13.0.0.19 Revised to use tabs mapping to page numbers.
version 13.0.0.20 Revised to permit a string to be used as a tab map
version 13.0.0.21 Revised to expand tab map with range desig ( i[23-64] )
version 13.0.0.22 Combines v12 and v13 -- compatible with both
version 13.0.0.23 Fixes bug causing MM "not unique" errors.
version 13.0.0.24 Now uses TNG database interface. // not released yet //
version 13.0.0.25 Bug fix - Error when displaying folio linked with family.
version 13.0.0.26 Fixed bookmarks & refactored code
*/

// showfolio is called with query string: showfolio.php?mediaID=327&index=6
if( empty( $_GET['mediaID' ] ) )
{
	header( "Location: thispagedoesnotexist.html" );
	exit;
}

$textpart = "showphoto";
$debug = false;
require 'tng_begin.php';

/*************************************************************************
LANGUAGE SUPPORT
*************************************************************************/
// include user-provided language support if any
if( file_exists( $mylanguage.'/folio_text.php' ) )
	include $mylanguage.'/folio_text.php';
else
	// otherwise default to this (English)
	include 'languages/English-UTF8/folio_text.php';

/*************************************************************************
GLOBAL PREFERENCES SET BY SHOWFOLIO MOD OPTIONS EDITOR
**************************************************************************/
$restricted = 0;
$centerpages = 1;
$showddmenus = 1;
$showinfo = 1;
$bordersize = 0;
$bordercolor = "gray";
$text_align = !empty( $centerpages ) ? 'center' : 'left';
$table_align = !empty( $centerpages ) ?"style='margin-left:auto;margin-right:auto;'" :'';

/*************************************************************************
WORKING LINKS
**************************************************************************/
$showfolio_url		 = getURL( 'showfolio', 1 );
$bookmarks_url		 = getURL( 'bookmarks', 1 );
$addbookmark_url		= getURL( "ajx_addbookmark", 1 );
$admin_editmedia_url = getURL( 'admin_editmedia', 1 );
$browsemedia_url		= getURL( 'browsemedia', 1 );
$home_url				= getURL( 'index', 0 );
$login_url				= getURL( 'login', 0 );
$home_icon				= $cms['tngpath'].'img/tng_home.gif';
$media_icon			 = $cms['tngpath'].'img/tng_media.gif';
$print_icon			 = $cms['tngpath'].'img/tng_print.gif';
$bookmark_icon		 = $cms['tngpath'].'img/tng_bmk.gif';

/* File types allowed as folio pages. */
$allowed = array(
	".jpg", ".JPG",
	".jpeg", ".JPEG",
	".gif", ".GIF",
	".png", ".PNG",
	".htm", ".HTM",
	".html", ".HTML",
	".php", ".PHP"
);

/* File types that are treated as images.*/
$imgList = array(
	".jpg", ".JPG",
	".jpeg", ".JPEG",
	".gif", ".GIF",
	".png", ".PNG"
);

/* TESTING ONLY - FORECE SESSION REFRESH */
if(!empty($ff)) unset( $_SESSION['folio'] ); // force recalc on each page

/*************************************************************************
RETRIEVE FOLIO DATA FROM MEDIA TABLE FOLDER AND FOLIO.PHP OPTIONS
**************************************************************************/
if( empty( $_SESSION['folio']['mediaID'] ) ||
						$mediaID != $_SESSION['folio']['mediaID'] )
{
	$fData = array();

	initMediaTypes();
 	$fData=get_fData( $mediaID, $mediatypes, $allowed );

	/* Override folio globals if requested in folio.php. */
	if( !isset($fData['restricted'] ) )
		$fData['restricted'] = $restricted;

	/* Set default for magnifier. */
	$fData['showmagnifier'] = 0;

	// force login if access is restricted
	if (!empty($fData['restricted']) && !$currentuser) header("Location: {$login_url}");

	unset( $_SESSION['folio'] );
	$_SESSION['folio'] = $fData;
}
else
{
	if( isset( $mag ) )
	{
		/* Toggle the magnifier. */
		$_SESSION['folio']['showmagnifier'] =
			empty( $_SESSION['folio']['showmagnifier'] ) ? 1 : 0;
	}
	$fData = $_SESSION['folio'];

}

/* The following are for Wordpress compatibility. */
$mediapath_rel = $fData['mediakey'];
$mediapath_abs = $rootpath.$mediapath_rel;
$dirname_rel = pathinfo( $mediapath_rel, PATHINFO_DIRNAME );
$dirname_abs = pathinfo( $mediapath_abs, PATHINFO_DIRNAME );

/* Define path for use with folio-relative images in html page files. */
define( "FOLIODIR", $dirname_rel.'/' );

$ID = $fData['mediatypeID'];

$header_icon = empty( $fData['icon'] ) ?
  	// use sprite
  	"<span class=\"headericon\" id=\"$ID-hdr-icon\"></span>" :
  	// use icon
  	"<img src=\"".$fData['icon']."\" width=\"20\" height=\"20\" alt=\"\" style=\"vertical-align:-2px\"	border=\"0\"/>";

/* Will the page have a tab line? - only if two or more pages.*/
$tabs = $fData['tabs'];
if( !empty( $tabs ) )
	$istabs = count( $tabs ) > 1 ? true : false;
else
	$istabs = false;

/*************************************************************************
NORMALIZE REQUESTED FOLIO PAGE INDEX
**************************************************************************/
/**
 **	A request from the tab number box.
 **	The $tabnr is a tab label that can be "looked up" in the tabs table
 **	or used as is if no tab map.
 **/
 if( $istabs && !empty( $tabnr ) )
 {
 	$index = tab2index( $fData, $tabnr );
 }

// Normalize index
$pageCount = $fData['pageCount'];
$index = isset( $index ) ? $index : 0;
$index = $index < 0 ? 0 : $index;
$index = $index > $pageCount - 1 ? $pageCount - 1 : $index;

/*************************************************************************
GET PAGE FILE NAME, SIZE AND EXTENSION TYPE
**************************************************************************/
$pageFile_abs = $rootpath.$tabs[$index]['file'];
$pageFile_rel = $tabs[$index]['file'];
$filesize = formatbytes($pageFile_abs, "KB");
$ext = strrchr($pageFile_abs, '.');

/*************************************************************************
CREATE HORIZONTAL PAGE TABS
*************************************************************************/
$tabstr = '';

// if only one page do not show tab line
if( $istabs )
{
	/*********************************************************************
	DETERMINE Inner TAB RANGE AND ELLIPSES
		First and last page tabs will always be displayed
		Inner tabs between them may require ellipsis before and/or after
	*********************************************************************/

	// number of inner tabs to show on either side of requested page (default=2)
	$sidetabs = isset($fData['sidetabs']) ? $fData['sidetabs'] : 2;

	// set maximum allowable Inner tabs
	$maxInnerTabs = $sidetabs * 2 + 1;

	// set actual number of inner tabs to lesser of $maxInnerTabs or $pageCount - 2
	$innerTabs =	$maxInnerTabs > $pageCount - 2 ? $pageCount - 2 : $maxInnerTabs;

	// set Inner index range
	$firstInnerIdx = $index - $sidetabs;
	$lastInnerIdx = $firstInnerIdx + $innerTabs - 1;

	// when requested index is tight right, move the Inner tabs left
	while( $lastInnerIdx > $pageCount - 2 )
	{
		$lastInnerIdx--;
		$firstInnerIdx--;
	}

	// when requested index is tight left, move the Inner tabs right
	while( $firstInnerIdx < 1 )
	{
		$firstInnerIdx++;
		$lastInnerIdx++;
	}

	// set the ellipses for missing sequence of tab numbers
	$frontEllipsis = $firstInnerIdx > 1 ? ". . ." : null;
	$backEllipsis = $lastInnerIdx < $pageCount - 2 ? ". . ." : null;

/*************************************************************************
ADD PREVIOUS PAGE BUTTON
*************************************************************************/
	$prev = $index - 1;
	if ( $index > 0 )
	{ // show live link
		$prevButton = "<a href=\"{$showfolio_url}mediaID={$fData['mediaID']}&index=$prev\" class=\"snlink\"
		title=\"{$text['fo_prevpage']}\">
		<font size='4'>&laquo;</font>&nbsp;{$text['fo_page']}</a>";
	}
	else
	{ // show dead link
		$prevButton = "<span class=\"snlink snlinkact\"
		title=\"{$text['fo_prevpage']}\">
			<font size='4'>&laquo;</font>&nbsp;{$text['fo_page']}</span>\n";
	}
	$tabstr .= "$prevButton\n";

	/*************************************************************************
	ADD FIRST ANCHOR TAB AND ELLIPSIS
		first and last file tabs are always displayed
	*************************************************************************/
	$tablabel = $tabs[0]['tab'];

	/* Set tool-tip rules for numeric vs text tab labels.*/
	if( is_numeric( $tablabel ) )
	{
		$page = "page";	// used in title (tooltip) - eg, "page 5"
		$fotab = "fotab"; // style sets tab width - eg, class="fotab"
	}
	else
	{
		$page = null;	// not needed for title (tooltip)
		$fotab = null;	// style not needed to set text tab width
	}

	// when first anchor tab is requested create simulated (inactive) link
	if( $index == 0 )
	{
	 $tabstr .= "<span class=\"snlink snlinkact $fotab\"
		title=\"$page $tablabel\">$tablabel</span>\n";
	}
	else
	{ // create live link - index always zero for first page file
		$tabstr .= "<a href=\"{$showfolio_url}mediaID={$fData['mediaID']}&index=0\"
			class=\"snlink $fotab\" title=\"$page $tablabel\">$tablabel</a>\n";
	}

/*************************************************************************
ADD THE Inner TABS
	Inner tabs lie between the first and last anchor tabs
	if the span is too large to show all tabs, use ellipses for missing tabs
*************************************************************************/
	$tabstr .= $frontEllipsis;

	for( $i = $firstInnerIdx; $i <= $lastInnerIdx; $i++ )
	{
		$tablabel = $tabs[$i]['tab'];

		// when current page is requested create simulated (inactive) link
		if( $i == $index )
		{
			// create dead link
			$tabstr .= "<span class=\"snlink snlinkact fotab\"
			title=\"page $tablabel\">$tablabel</span>\n";
		}
		else
		{
			// create live link
			$tabstr .= "<a href=\"{$showfolio_url}mediaID={$fData['mediaID']}&index=$i\"
				class=\"snlink fotab\" title=\"page $tablabel\">$tablabel</a>\n";
		}
	}
	$tabstr .= $backEllipsis;

/*************************************************************************
ADD THE LAST ANCHOR TAB
*************************************************************************/
	$lastIndex = $pageCount -1;
	$tablabel = $tabs[$lastIndex]['tab'];

	if( $index == $lastIndex )
	{
		// create dead link
	 $tabstr .= "<span class=\"snlink snlinkact fotab\"
		title=\"page $tablabel\">$tablabel</span>\n";
	}
	else
	{
		// create live link
		$tabstr .= "<a href=\"{$showfolio_url}mediaID={$fData['mediaID']}&index=$tablabel\" class=\"snlink fotab\"
		title=\"page $tablabel\">$tablabel</a>\n";
	}

/*************************************************************************
ADD NEXT PAGE BUTTON
*************************************************************************/
	$next = $index + 1;
	if ( $index < $pageCount - 1 )
	{
		$nextButton="<a href=\"{$showfolio_url}mediaID={$fData['mediaID']}&index=$next\" class=\"snlink\"
		title=\"{$text['fo_nextpage']}\">{$text['fo_page']}&nbsp;<font size='4'>&raquo;</font></a>";
	}
	else
	{
		// show dead link
		$nextButton = "<span class=\"snlink snlinkact\" title=\"{$text['fo_nextpage']}\">{$text['fo_page']}&nbsp;<font size='4'>&raquo;</font></span>\n";
	}
	$tabstr .= $nextButton;

/*************************************************************************
CREATE A TAB SELECTION BOX
*************************************************************************/
	$formstr = getFORM( "showfolio", "GET", null, null, null );
	$tabselector = "
$formstr &nbsp;{$text['fo_entertab']}&nbsp;
<input type='hidden' name='mediaID' value={$fData['mediaID']} />
<input type=\"text\" size=\"5\" name=\"tabnr\" />
<input type=\"submit\" value=\"&lt;--\" class=\"none\" />
</form>";
} // if $istabs - tab string has been constructed

/*************************************************************************
CONSTRUCT A COMPLETE TAB LINE FROM ITS ELEMENTS
*************************************************************************/
/* Use cells in a one-row table to vertically align all the tab line elements.*/
if( !empty( $tabstr ) || !empty( $fData['pdfhref'] ) )
{
	$outstr	= "
<table $table_align>
<tr>";
	if( !empty( $tabstr ) )
	{
		$outstr .= "
	<td>
		$tabstr
	</td>
	<td nowrap=\"nowrap\" style='vertical-align:bottom;'>
		$tabselector
	</td>";
	}

	// add PDF button if there is a download
	if( !empty($fData['pdflink'] ) )
	{
		$pdfhref = $dirname_rel .'/'. $fData['pdflink'];
	}
	elseif( !empty($fData['pdffetch'] ) )
	{
		$pdfhref = 'pdffetch.php?pdf=' . $fData['pdffetch'];
	}

	if( !empty( $pdfhref ) )
	{
		$outstr .= "
	<td width='5'>&nbsp;</td>
	<td nowrap=\"nowrap\" style='vertical-align:bottom;padding-bottom:2px;'>
		<a style=\"float:right\" href=\"$pdfhref\" class=\"snlink\" title=\"{$text['fo_downloadpdf']}\">PDF</a>
	</td>
	<td width='5'>&nbsp;</td>";

	}
	$outstr .= "
</tr>
</table>
<br style='clear:both' />
";
}

/**********************************************************
MAKE TNG LOG ENTRY - SET BOOKMARK
***********************************************************/
$title = $fData['description'];

// Program to display URL of current page.
if (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on')
    $loglink = "https";
else
      $loglink = "http";

// Here append the common URL characters.
$loglink .= "://";

// Append the host(domain name, ip) to the URL.
$loglink .= $_SERVER['HTTP_HOST'];

// Append the requested resource location to the URL
$loglink .= $_SERVER['REQUEST_URI'];

$tablabel = $tabs[$index]['tab'];
$title .= is_numeric( $tablabel ) ? ', page '.$tablabel : ', '.$tablabel.= ' page';

// log and bookmark the document page number
$logstring = "<a href=\"$loglink\">$title</a>";


# If you dont want image views logged, comment out the next line
writelog($logstring);
preparebookmark($logstring);

/**********************************************************
CREATE HTML DOCUMENT HEADER
***********************************************************/

$flags['scripting'] = "
<style type = \"text/css\" >
.fotab {
	display:inline-block;
	min-width:2em;
	text-align:center;
}
</style>
";
$flags['noheader'] = false;
$flags['nobody'] = false;
$flags['noicons'] = $showddmenus ? false : true;
tng_header( $title, $flags );

/*************************************************************************
DISPLAY AN ALBUM STYLE HEADER
*************************************************************************/
$header = $fData['description'];

$header .= "&nbsp;<a href=\"?mediaID=$mediaID&amp;index=$index&amp;mag=1\" title=\"{$text['fo_showmag']}\"><img src=\"{$cms['tngpath']}img/tng_find.gif\" border=\"0\"></a>";

if ($allow_edit && empty( $tngprint ) )
{
	$header .= "&nbsp;<a href=\"{$admin_editmedia_url}mediaID=$mediaID&amp;cw=1\" target=\"_blank]\" title=\"{$text['editmedia']}\"><img src=\"{$cms['tngpath']}img/tng_aedit.gif\" border=\"0\"></a>";
}
if( !$showddmenus && empty( $tngprint ) )
{
	echo "
<div style=\"display:table;width:65%;text-align:center;\">
<div style=\"display:table-row\">

<div style=\"display:table-cell;width:25%;\">
<div style=\"vertical-align:bottom;\">
	&nbsp;<img src=\"$home_icon\" width=\"16\" style=\"vertical-align:bottom;\"/>
	&nbsp;<a href=\"$home_url\">{$text['homepage']}</a>
</div>
</div>

<div style=\"display:table-cell;width:25%;text-align:center;\">
<div style=\"vertical-align:bottom;\">
	&nbsp;<img src=\"$media_icon\" width=\"16\"style=\"vertical-align:bottom;\"/>
	&nbsp;<a href=\"{$browsemedia_url}mediatypeID=$ID\">".ucfirst( $ID )."</a>
</div>
</div>

<div style=\"display:table-cell;width:25%;text-align:center;\">
<div style=\"vertical-align:bottom;\">
	&nbsp;<img src=\"$print_icon\" width=\"16\" style=\"vertical-align:bottom;\"/>";

if( !$tngconfig['showprint'] && $sitever != "mobile")
	{
		if( $text['tngprint'] == 'Print' ) $text['tngprint'] = 'Printer friendly';
		echo "
&nbsp;<a href=\"{$showfolio_url}mediaID=$mediaID&amp;tngprint=1\">{$text['tngprint']}</a>";
	}

echo "
</div>
</div>

<div style=\"display:table-cell;width:25%;text-align:center;\">
<div style=\"vertical-align:bottom;\">
	&nbsp;<img src=\"$bookmark_icon\" width=\"16\" style=\"vertical-align:bottom;\"/>
	&nbsp;<a href=\"$bookmarks_url\" onclick=\"tnglitbox = new LITBox('{$addbookmark_url}p=" . urlencode($cms['tngpath']) . "',{width:350,height:100}); return false;\">{$text['bookmark']}</a>
</div>
</div>

</div>
</div>
<br />";
}

echo "
<table $table_align cellpadding=\"0\" cellspacing=\"0\">
<tr>
<td valign=\"top\">
<a href=\"{$browsemedia_url}mediatypeID=$ID\" title=\"{$text['showall']} {$fData['display']}\">$header_icon</a>
</td>
<td>
<h1 class=\"header\" style=\"margin-bottom:0px; margin-left:5px;\">$header</h1>
<span class=\"normal\" style=\"margin-left:5px;\">{$fData['notes']}</span>
</td>
</tr>
</table>
<br style='clear:both;' /><br />";
if( isset( $outstr ) && empty( $tngprint ) ) echo $outstr;

/**********************************************************
DISPLAY CONTENT OF PAGE FILE
***********************************************************/
// Display image as page file
if ( in_array( $ext, $imgList ) )
{
	list ($width, $height) = getimagesize($pageFile_abs);
	$imgSize = "$width x $height";
	echo "
<div style='text-align:$text_align;min-height:1280;min-width:680;'>
<img src=\"$pageFile_rel\" style=\"max-width:100%;margin-top:5px; border:{$bordersize}px solid {$bordercolor};\" onmouseover=\"TJPzoomif(this);\" id=\"pageImg\" alt=\"Document\" />
</div>";
}
// Display html or php page file
else {
	$imgSize = "n/a";
	if ( !empty( $pageFile_abs ) )
	{
		if( $ext == '.htm' || $ext == '.html' || $ext == '.php' )
		{
			show_webpage( $pageFile_rel );
		}
	}
}

/**********************************************************
DISPLAY META DATA AT BOTTOM OF PAGE
***********************************************************/
echo "<hr />
<table border=\"0\" cellspacing=\"1\" cellpadding=\"4\" width=\"100%\" class=\"whiteback\">
<tr>
<td style=\"width:100px\" class=\"fieldnameback fieldname\">
{$text['photoowner']}
</td>
<td class=\"databack\">
{$fData['owner']}
</td>
</tr>
";
if( !empty( $showinfo ) )
{
	echo "<tr>
<td style=\"width:100px\" class=\"fieldnameback fieldname\">
{$text['filename']}
</td>
<td class=\"databack\">
$pageFile_rel
</td>
</tr>

<tr>
<td style=\"width:100px\" class=\"fieldnameback fieldname\">
{$text['filesize']}
</td>
<td class=\"databack\">
$filesize
</td>
</tr>

<tr>
<td style=\"width:100px\" class=\"fieldnameback fieldname\">
{$text['fo_mediaid']}
</td>
<td class=\"databack\">
$mediaID
</td>
</tr>

<tr>
<td style=\"width:100px\" class=\"fieldnameback fieldname\">
{$text['photosize']}
</td>
<td class=\"databack\">
$imgSize
</td>
</tr>

<tr>
<td style=\"width:100px\" class=\"fieldnameback fieldname\">
Folio version
</td>
<td class=\"databack\">
$app_version ($app_revdate)
</td>
</tr>
";
}
echo "<tr>
<td style=\"width:100px\" class=\"fieldnameback fieldname\">
{$text['indlinked']}
</td>
<td class=\"databack\">
{$fData['linkStr']}
</td>
</tr>";

if( !empty( $pdfhref ) && empty( $tngprint ) )
{
	echo "
<tr>
<td style=\"width:100px\" class=\"fieldnameback fieldname\">
{$text['fo_downloadpdf']}
</td>
<td class=\"databack\">
<a href=\"$pdfhref\" >{$fData['description']}</a>, {$fData['notes']}
</td>
</tr>";
}
echo "
</table>";

if( empty( $tngprint ) )
{
echo "
<table>
<tr>
<td style=\"width:280px;\">
<a href=\"javascript:scroll(0,0)\" target=\"_self\" class=\"snlink\" >{$text['fo_backtotop']}</a>
";
if ( $index > 0)
	echo	"&nbsp;
$prevButton
";
if ( $index < $pageCount - 1 )
	echo "&nbsp;
$nextButton
";

echo "</td>
</tr>
</table>";
}

if( !empty( $fData['showmagnifier'] ) )
{
	echo "
<script type=\"text/javascript\" src=\"js/ddpowerzoomer.js\"></script>
<script>
jQuery(document).ready(function($){ //fire on DOM ready
 $('#pageImg').addpowerzoom({
	defaultpower: 2,
	powerrange: [2,2],
	largeimage: null,
	magnifiersize:[400,100]
 })
})
$ = jQuery; // added to prevent lightbox.js code failing
</script>
";
}
tng_footer( "" );

/*************************************************************************
	FUNCTIONS
*************************************************************************/
function get_fData( $mediaID, $mediatypes, $allowed )
{
	$fData['mediaID'] = $mediaID;

	if( false == $fData = addFolioRecord( $fData ) )
	{
		trigger_error( "Folio with ID $mediaID is invalid", E_USER_ERROR );
		exit;
	}

	$fData = addFolioOptions( $fData );

	initMediaTypes();
	$fData = addMediaTypes( $fData, $mediatypes );

	$fData = addLinkString( $fData );

	$fData = addSortedPagefileNames( $fData, $allowed );

	$fData = assignTabLabels( $fData );

	return $fData;
} // get_fData

function addFolioRecord ( $fData )
{
	global $media_table;

	$mediaID = $fData['mediaID'];

  /* Get folio data from media_table.*/
  $query = "SELECT mediaID, mediatypeID, mediakey, gedcom, path, notes, owner,
			description from $media_table WHERE mediaID=$mediaID";
  $result = tng_query( $query );
  $row = tng_fetch_assoc( $result );

	if( empty( $row ) )
	{
		return false; // invalid mediaID
	}

	$mediakey = $row['mediakey'];
	if( false === strpos( $mediakey, 'folio.php' ) )
	{
		return false; // not a folio
	}

	/* Add db record query data to fData set */
	foreach( $row as $key => $value )
	  $fData[$key] = $value;

	return $fData;
} //getFolioRecord

/* Add all settings from folio.php to the fData array. */
function addFolioOptions( $fData )
{
	global $media_table;

	$pdffetch_url	= getURL( "pdffetch", 1 );

	$mediakey = $fData['mediakey'];
	$foliodir_rel = dirname( $mediakey );

  // Cannot "include" folio.php because it contains a redirection
  $buffer = file_get_contents( $mediakey ); // path to folio
	preg_match_all("#^\s*\\$([a-z]+)=(.+);#m", $buffer, $matches, PREG_SET_ORDER );
	if( !empty( $matches ) )
	{
		foreach( $matches as $match )
		{
			$fData[$match[1]] = trim($match[2], "\'\"");
		}
	}
/*
	if( !empty( $fdata['pdffetch'] ) )
	{
		$pdffetch_url = getURL( "pdffetch", 1 );

  	// use pdffetch to download file from protected location
  	$fData['pdfhref'] = $pdffetch_url.'pdf='.$fData['pdffetch'];
  }
  elseif( !empty( $fData['pdflink'] ) )
  {
  	// use direct link to download pdf file relarive to TNG root
  	$fData['pdfhref'] = $foliodir_rel.'/'.$fData['pdflink'];
	}
*/
	return $fData;
} // addFolioOptions

/* Any mediatype can contain folios - folios can be idenfitied
** in the databse because the media file for a folio is 'folio.php';
*/
function addMediaTypes( $fData, $mediatypes )
{
	$ID = $fData['mediatypeID'];

  foreach( $mediatypes as $mediatype )
  {
  	if( $mediatype['mediatypeID'] == $ID )
  	{
			/* Text[ID] allows language support override for the folio display title.*/
		  if (isset($text[$ID]))
		 		$display = $text[$ID];
		  else
		  	$display = $mediatype['display'];

			$fData['display'] = $display;
  		$fData['collpath'] = $mediatype['path'];
			$fData['icon'] = $mediatype['icon'];
  		break;
  	}
  }
	return $fData;
}

function addLinkString( $fData ) {
	global $medialinks_table;

	$getperson_url		 = getURL( 'getperson', 1 );
	$familygroup_url		= getURL( 'familygroup', 1 );
	$placesearch_url		= getURL( 'placesearch', 1 );
	$showsource_url		= getURL( 'showsource', 1 );
	$showrepo_url			= getURL( 'showrepo', 1 );

  $medialinks = array();
	$mediaID = $fData['mediaID'];

  $query = "SELECT * FROM $medialinks_table WHERE mediaID = '$mediaID' ORDER BY linktype";

  $result = tng_query( $query );
  while( $row = tng_fetch_assoc( $result ) )
  {
    $medialinks[] = $row;
  }

  $linkStr = "";
  foreach($medialinks as $medialink)
  {
  	$personID = $medialink['personID'];
  	$tree = $medialink['gedcom'];
  	switch ($medialink['linktype'])
  	{
  		case 'I':
  			$linkname = getPersonName($personID, $tree);
  			$linkStr .= "<a href =\"{$getperson_url}personID=$personID&amp;tree=$tree\">$linkname</a>; ";
  			break;
  		case 'F':
  			$linkname = getCoupleName($personID, $tree);
  			$linkStr .= "<a href =\"{$familygroup_url}familyID=$personID&amp;tree=$tree\">$linkname</a>; ";
  			break;
  		case 'L':
  			$linkname = $personID;
  			$linkStr .= "<a href =\"{$placesearch_url}psearch=$personID&amp;tree=$tree\">$linkname</a>; ";
  			break;
  		case 'S':
  			$linkname= getSourceName($personID, $tree);
  			$linkStr .= "<a href =\"{$showsource_url}sourceID=$personID&amp;tree=$tree\">$linkname</a>; ";
  			break;
  		case 'R':
  			$linkname= getRepoName($personID, $tree);
  			$linkStr .= "<a href =\"{$showrepo_url}repoID=$personID&amp;tree=$tree\">$linkname</a>; ";
  			break;
  		default:
  			die (__LINE__.": unrecognized media link type");
  			break;
  	} // switch linktype
  } // foreach $medialinks

  $fData['linkStr'] = substr($linkStr, 0, -2);

	return $fData;
}

function addSortedPagefileNames( $fData, $allowed ) // tabs
{
	$exclude = empty ( $fData['exclude'] ) ? array() : $fData['exclude'];

  $fileNames = array();
	$path = $fData['mediakey'];
	$dirname_rel = dirname( $path );
	$dirname_abs = __DIR__ . '/' . $dirname_rel;

  if( $handle = opendir( $dirname_abs ) )
  {
  	while( false !== ( $file = readdir($handle ) ) )
  	{
  		strtolower( $file );
  		// these files will always be exluded as page files
  		if( $file == "folio.php" ) continue;
  		if( $file == "index.php" ) continue; // backward compatibility
  		if( $file == "init.php" ) continue; // backward compatibility
  		$ext = strrchr( $file, '.' );
  		if( in_array( $ext, $allowed ) )
  			if( !in_array( $ext, $exclude ) )
				{
  				$fileNames[] = $dirname_rel.'/'.$file;
				}
		}
  	closedir($handle);
  }

  //print_r( $fileNames );exit;
  $pageCount=count($fileNames);
  if ($pageCount > 1)
  {
  	sort($fileNames);
  }

	if( !empty( $fData['usecover'] ) )
	{
		$offset = 2;
	}
	else
	{
		$offset = 1;
	}

	for($i=0;isset( $fileNames[$i]); $i++)
	{
		$tabs[$i]['file'] = $fileNames[$i];
		$tabs[$i]['tab'] = $i+$offset;
  }
	$fData['tabs'] = $tabs;
	$fData['pageCount'] = $pageCount;
	return $fData;
} // addSortedPagefileNames

/* Add tab labels to page files in $tabs array based on $tabmap and $usecover. */
function assignTabLabels( $fData )
{
	global $text; // For custom language support

	/* A tab map overrides a usecover request.  So if no tab map
	** is defined but we have $usecover, set tab[0]['tab'] and return.
	** If $button label not defined, use language support; if no
	** language support, just use 'Cover' as the tab[0] label.*/
	if( empty($fData['tabmap'] ) )
	{
		if( !empty( $fData['usecover'] ) )
		{
			/* Make room for the cover page by adjusting numbered page labels
			** to start with '1' at index=2, right after Cover page. */
			for($i=1; isset($fData['tabs'][$i]['tab']);$i++ )
			{
				$fData['tabs'][$i]['tab'] = $i;
			}

			/* Set index=0 to the cover page label. */
			if( !empty( $fData['buttonlabel'] ) )
			{
				$buttonlabel = $fData['buttonlabel'];
				$fData['tabs'][0]['tab'] = $buttonlabel;
			}
			else
			{
				$fData['tabs'][0]['tab'] = !empty( $text['fo_cover'] ) ? $text['fo_cover'] : 'Cover';
			}
			return( $fData );
		}
		return $fData;
	}

	/* Reset the tab labels per the tab map */
  $istabs = true;
	$tabmap = $fData['tabmap'];
	$pageCount = $fData['pageCount'];

	/* Turn tabmap string into array if not already so. */
	if( !is_array( $tabmap ) )
 	{
 		// The $tabmap is given as a string - convert to array
 		$temp = explode( ",", $tabmap );
 		$tabmap = array();
 		foreach( $temp as $value )
 		{
 			$tabmap[] = trim($value);
 		}
 	}

 	// Expand tab map fill ranges ( [x-y] ) with sequential numbers
 	for( $i=0; isset($tabmap[$i]); $i++ )
 	{
 		// Expand range in this format '[42-324]' with incrementing numbers
 		if( preg_match( "#([a-zA-Z]?)\[([0-9]+)-([0-9]+)\]#", $tabmap[$i], $matches ) )
 		{
 			// Create array from the range
 			$j = $matches[2];
 			while( $j <= $pageCount && $j <= $matches[3] )
 			{
 				$range_arr[] = $matches[1].$j++;
 			}

 			// Replace the range notation with expanded range array
 			array_splice( $tabmap, $i, 1, $range_arr );
 			$range_arr = array();
 		}
 	}

	/* Transfer expanded tab map to the tabs array.*/
	$tabs = $fData['tabs'];
	for($i=0; isset($tabs[$i]); $i++)
	{
		$tabs[$i]['tab'] = $tabmap[$i];
	}

	$fData['tabs'] = $tabs;
  return $fData;
} // expandTabMap

function tab2index( $fData, $TAB )
{
	/**
	 **	A request from the tab number box.
	 **	The $tabnr is a tab label that can be "looked up" in the tabs array
	 **	or used as-is if no tab map.
	 **/
	$tabs = !empty( $fData['tabs'] ) ? $fData['tabs'] : 0;


	//LOOK FOR TABNR, FIRST AS PAGE NR, THEN AS START OF TEXT LABEL
	while (!empty( $TAB ) )
	{
		// check as numeric == tab
		if( is_numeric( $TAB ) )
		{
			for( $i=0; isset($tabs[$i]); $i++ )
			{
				if( !is_numeric( $tabs[$i]['tab'] ) ) continue;
				if( $TAB == $tabs[$i]['tab'] )
					break 2;
			}
		}

		// check to see if requested tab starts a string label (numbers and/or chars)
		for( $i=0; isset($tabs[$i]); $i++ )
		{
			if( is_numeric( $tabs[$i]['tab'] ) ) continue;
			if( false !== $pos = stripos( $tabs[$i]['tab'], $TAB, 0 ) )
			{
				if( $pos == 0 ) break 2;
			}
		}

		// NOT FOUND ANYWHERE
		$i=0;
		break;
	}
return $i;
} // tab2index

function getRepoName($sourceID, $tree)
{
global $repositories_table;
	$query = "SELECT * FROM $repositories_table WHERE repoID = '$sourceID' AND gedcom='$tree' ";

  $result = tng_query( $query );
  $row = tng_fetch_assoc( $result );
	if( !empty( $row ) )
		return $rows['reponame'];
	else
		return '';
}

function getSourceName ($sourceID, $tree)
{
global $sources_table;
	$query = "SELECT * FROM $sources_table WHERE sourceID = '$sourceID' AND gedcom='$tree' ";

  $result = tng_query( $query );
  $row = tng_fetch_assoc( $result );
	if( !empty( $row ) )
		return $row['title'];
	else
		return '';
}

function getCoupleName($familyID, $tree)
{
global $families_table;
	$query = "SELECT * FROM $families_table WHERE familyID = '$familyID' AND gedcom='$tree' ";
	//$rows = db_select( $query );

  $result = tng_query( $query );
  $row = tng_fetch_assoc( $result );

	if( !empty( $row ) ) {
		$husband = $row['husband'];
		$wife = $row['wife'];
		$name = getPersonName($husband, $tree) . "/" . getPersonName($wife, $tree);
		return $name;
	}
	return '';
}

function getPersonName($personID, $tree)
{
global $people_table;
	$name = "";
	$query = "SELECT gedcom, firstname, lastname FROM $people_table WHERE personID = '$personID' AND gedcom ='$tree' ";

  $result = tng_query( $query );
  $row = tng_fetch_assoc( $result );
	if( !empty( $row ) ) {
		$name = $row['firstname'] . " " . $row['lastname'];
		return $name;
	}
	return '';
}

function formatbytes($file, $type)
{
	switch($type)
	{
		case "KB":
			$filesize = filesize($file) * .0009765625; // bytes to KB
			break;
		case "MB":
			$filesize = (filesize($file) * .0009765625) * .0009765625; // bytes to MB
			break;
		case "GB":
			$filesize = ((filesize($file) * .0009765625) * .0009765625) * .0009765625; // bytes to GB
			break;
	}
	if($filesize <= 0)
		return $filesize = 'unknown';
	else
		return round($filesize, 2).' '.$type;
}

/* Resolve image paths in page files -- check folder-relative first
** if no file exists, accept path as tng-relative (no action required) */
function show_webpage( $pagefile_abs )
{
	$contents = file_get_contents( $pagefile_abs );
	preg_match_all( "#src\s*=\s*[\"\'\\\]*([^\"\'\\\]*)#", $contents, $matches, PREG_SET_ORDER );
	for( $i=0; isset($matches[$i][1]); $i++ )
	{
		$ipath = FOLIODIR.$matches[$i][1];

		// if folder-relative file, replace with full root-relative path
		if( file_exists( $ipath ) )
		{
			$contents = str_replace( $matches[$i][1], $ipath, $contents );
		}
		// otherwise assume path is already root-relative and do nothing
	}
	// display the page file
	eval( '?>'.$contents );
}
